@@ -7,6 +7,7 @@ import os
 import re
 import sys
 
+import traitlets
 from IPython.core.application import base_aliases as base_ip_aliases
 from IPython.core.application import base_flags as base_ip_flags
 from IPython.core.application import BaseIPythonApplication
@@ -23,6 +24,15 @@ from traitlets.config.application import LevelFormatter
 
 from .._version import __version__
 
+# FIXME: CUnicode is needed for cli parsing
+# with traitlets 4
+# bump when we require traitlets 5, which requires Python 3.7
+if int(traitlets.__version__.split(".", 1)[0]) < 5:
+    from traitlets import CUnicode
+else:
+    # don't need CUnicode with traitlets 4
+    CUnicode = Unicode
+
 # -----------------------------------------------------------------------------
 # Module errors
 # -----------------------------------------------------------------------------
@@ -100,7 +110,7 @@ class BaseParallelApplication(BaseIPythonApplication):
         '', config=True, help="The ZMQ URL of the iplogger to aggregate logging."
     )
 
-    cluster_id = Unicode(
+    cluster_id = CUnicode(
         '',
         config=True,
         help="""String id to add to runtime files, to prevent name collisions when
@@ -132,6 +142,8 @@ class BaseParallelApplication(BaseIPythonApplication):
     def initialize(self, argv=None):
         """initialize the app"""
         super(BaseParallelApplication, self).initialize(argv)
+        if "IPP_SESSION_KEY" in os.environ:
+            self.config.Session.key = os.environ["IPP_SESSION_KEY"].encode("ascii")
         self.init_deprecated_config()
         self.to_work_dir()
         self.reinit_logging()
