@@ -1,37 +1,16 @@
 """
-A simple logger object that consolidates messages incoming from ipcluster processes.
-
-Authors:
-
-* MinRK
-
+A logger object that consolidates messages incoming from ipcluster processes.
 """
-
-#-----------------------------------------------------------------------------
-#  Copyright (C) 2011  The IPython Development Team
-#
-#  Distributed under the terms of the BSD License.  The full license is in
-#  the file COPYING, distributed as part of this software.
-#-----------------------------------------------------------------------------
-
-#-----------------------------------------------------------------------------
-# Imports
-#-----------------------------------------------------------------------------
-
-
 import logging
-import sys
 
 import zmq
-from zmq.eventloop import ioloop, zmqstream
-
-from traitlets.config.configurable import LoggingConfigurable
 from jupyter_client.localinterfaces import localhost
-from traitlets import Int, Unicode, Instance, List
-
-#-----------------------------------------------------------------------------
-# Classes
-#-----------------------------------------------------------------------------
+from traitlets import Instance
+from traitlets import List
+from traitlets import Unicode
+from traitlets.config.configurable import LoggingConfigurable
+from zmq.eventloop import ioloop
+from zmq.eventloop import zmqstream
 
 
 class LogWatcher(LoggingConfigurable):
@@ -42,10 +21,13 @@ class LogWatcher(LoggingConfigurable):
     """
 
     # configurables
-    topics = List([''], config=True,
-        help="The ZMQ topics to subscribe to. Default is to subscribe to all messages")
-    url = Unicode(config=True,
-        help="ZMQ url on which to listen for log messages")
+    topics = List(
+        [''],
+        config=True,
+        help="The ZMQ topics to subscribe to. Default is to subscribe to all messages",
+    )
+    url = Unicode(config=True, help="ZMQ url on which to listen for log messages")
+
     def _url_default(self):
         return 'tcp://%s:20202' % localhost()
 
@@ -53,10 +35,12 @@ class LogWatcher(LoggingConfigurable):
     stream = Instance('zmq.eventloop.zmqstream.ZMQStream', allow_none=True)
 
     context = Instance(zmq.Context)
+
     def _context_default(self):
         return zmq.Context.instance()
 
     loop = Instance('tornado.ioloop.IOLoop')
+
     def _loop_default(self):
         return ioloop.IOLoop.current()
 
@@ -82,36 +66,34 @@ class LogWatcher(LoggingConfigurable):
             self.stream.setsockopt(zmq.SUBSCRIBE, '')
         else:
             for topic in self.topics:
-                self.log.debug("Subscribing to: %r"%(topic))
+                self.log.debug("Subscribing to: %r" % (topic))
                 self.stream.setsockopt(zmq.SUBSCRIBE, topic)
-    
+
     def _extract_level(self, topic_str):
         """Turn 'engine.0.INFO.extra' into (logging.INFO, 'engine.0.extra')"""
         topics = topic_str.split('.')
-        for idx,t in enumerate(topics):
+        for idx, t in enumerate(topics):
             level = getattr(logging, t, None)
             if level is not None:
                 break
-        
+
         if level is None:
             level = logging.INFO
         else:
             topics.pop(idx)
-        
+
         return level, '.'.join(topics)
-            
-            
+
     def log_message(self, raw):
         """receive and parse a message, then log it."""
         if len(raw) != 2 or '.' not in raw[0]:
-            self.log.error("Invalid log message: %s"%raw)
+            self.log.error("Invalid log message: %s" % raw)
             return
         else:
             topic, msg = raw
             # don't newline, since log messages always newline:
-            topic,level_name = topic.rsplit('.',1)
-            level,topic = self._extract_level(topic)
+            topic, level_name = topic.rsplit('.', 1)
+            level, topic = self._extract_level(topic)
             if msg[-1] == '\n':
                 msg = msg[:-1]
             self.log.log(level, "[%s] %s" % (topic, msg))
-
