@@ -1,35 +1,50 @@
 """joblib parallel backend for IPython Parallel"""
 # Copyright (c) IPython Development Team.
 # Distributed under the terms of the Modified BSD License.
-
 from __future__ import absolute_import
 
+from joblib.parallel import AutoBatchingMixin
+from joblib.parallel import ParallelBackendBase
+
 import ipyparallel as ipp
-from joblib.parallel import ParallelBackendBase, AutoBatchingMixin
 
 
 class IPythonParallelBackend(AutoBatchingMixin, ParallelBackendBase):
     def __init__(self, view=None, **kwargs):
-        super(IPythonParallelBackend, self).__init__(**kwargs)
+        super().__init__(**kwargs)
+        self._cluster_owner = False
+        self._client_owner = False
         if view is None:
-            self._owner = True
-            rc = ipp.Client()
+            self._client_owner = True
+            try:
+                # load the default cluster
+                cluster = ipp.Cluster.from_file()
+            except FileNotFoundError:
+                # other load errors?
+                cluster = self._cluster = ipp.Cluster()
+                self._cluster_owner = True
+                cluster.start_cluster_sync()
+            else:
+                # cluster running, ensure some engines are, too
+                if not cluster.engines:
+                    cluster.start_engines_sync()
+            rc = cluster.connect_client_sync()
+            rc.wait_for_engines(cluster.n or 1)
             view = rc.load_balanced_view()
+
             # use cloudpickle or dill for closures, if available.
             # joblib tends to create closures default pickle can't handle.
             try:
-                import cloudpickle
+                import cloudpickle  # noqa
             except ImportError:
                 try:
-                    import dill
+                    import dill  # noqa
                 except ImportError:
                     pass
                 else:
                     view.client[:].use_dill()
             else:
                 view.client[:].use_cloudpickle()
-        else:
-            self._owner = False
         self._view = view
 
     def effective_n_jobs(self, n_jobs):
@@ -38,8 +53,10 @@ class IPythonParallelBackend(AutoBatchingMixin, ParallelBackendBase):
 
     def terminate(self):
         """Close the client if we created it"""
-        if self._owner:
+        if self._client_owner:
             self._view.client.close()
+        if self._cluster_owner:
+            self._cluster.stop_cluster_sync()
 
     def apply_async(self, func, callback=None):
         """Schedule a func to be run"""
