@@ -18,6 +18,7 @@ from decorator import decorator
 from IPython import get_ipython
 from IPython.display import display
 from IPython.display import display_pretty
+from IPython.display import publish_display_data
 
 from .futures import MessageFuture
 from .futures import multi_future
@@ -43,7 +44,7 @@ def check_ready(f, self, *args, **kwargs):
     """Check ready state prior to calling the method."""
     self.wait(0)
     if not self._ready:
-        raise error.TimeoutError("result not ready")
+        raise TimeoutError("result not ready")
     return f(self, *args, **kwargs)
 
 
@@ -64,6 +65,8 @@ class AsyncResult(Future):
     _tracker = None
     _single_result = False
     owner = False
+    _last_display_prefix = ""
+    _stream_trailing_newline = True
 
     def __init__(
         self,
@@ -138,20 +141,46 @@ class AsyncResult(Future):
         self._output_future.add_done_callback(self._resolve_output)
         self.add_done_callback(self._finalize_result)
 
-    def _iopub_streaming_output_callback(self, eid, msg):
+    def _iopub_streaming_output_callback(self, eid, msg_future, msg):
         """Callback registered during AsyncResult.stream_output()"""
         msg_type = msg['header']['msg_type']
+        ip = get_ipython()
+        if ip is not None:
+            in_kernel = getattr(ip, 'kernel', None) is not None
+        else:
+            in_kernel = False
+
         if msg_type == 'stream':
             msg_content = msg['content']
             stream_name = msg_content['name']
-            stream = getattr(sys, stream_name, sys.stdout)
-            self._display_stream(
-                msg_content['text'],
-                f'[{stream_name}:{eid}] ',
-                file=stream,
-            )
 
-        if get_ipython() is None:
+            if in_kernel:
+                parent_msg_id = msg['parent_header']['msg_id']
+                display_id = f"{parent_msg_id}-{stream_name}"
+                md = msg_future.output.metadata
+                full_stream = md[stream_name]
+                if display_id in self._already_streamed:
+                    update = True
+                else:
+                    self._already_streamed[display_id] = True
+                    update = False
+                publish_display_data(
+                    {
+                        "text/plain": f"[{stream_name}:{eid}] " + full_stream,
+                    },
+                    transient={"display_id": display_id},
+                    update=update,
+                )
+                return
+            else:
+                stream = getattr(sys, stream_name, sys.stdout)
+                self._display_stream(
+                    msg_content['text'],
+                    f'[{stream_name}:{eid}] ',
+                    file=stream,
+                )
+
+        if ip is None:
             return
 
         if msg_type == 'display_data':
@@ -168,14 +197,21 @@ class AsyncResult(Future):
 
         # Keep a handle on the futures so we can remove the callback later
         future_callbacks = {}
+        self._already_streamed = {}
+        self._stream_trailing_newline = True
+        self._last_display_prefix = ""
         for eid, msg_future in zip(self._targets, self._children):
-            callback_func = partial(self._iopub_streaming_output_callback, eid)
+            callback_func = partial(
+                self._iopub_streaming_output_callback, eid, msg_future
+            )
             future_callbacks[msg_future] = callback_func
             msg_future.iopub_callbacks.append(callback_func)
 
         try:
             yield
         finally:
+            # clear stream cache
+            self._already_streamed = {}
             # Remove the callback
             for msg_future, callback in future_callbacks.items():
                 msg_future.iopub_callbacks.remove(callback)
@@ -236,11 +272,11 @@ class AsyncResult(Future):
                 else:
                     raise e
         else:
-            raise error.TimeoutError("Result not ready.")
+            raise TimeoutError("Result not ready.")
 
     def _check_ready(self):
         if not self.ready():
-            raise error.TimeoutError("Result not ready.")
+            raise TimeoutError("Result not ready.")
 
     def ready(self):
         """Return whether the call has completed."""
@@ -430,12 +466,12 @@ class AsyncResult(Future):
                 # Event doesn't like timeout < 0
                 timeout = None
             elif timeout == 0:
-                raise error.TimeoutError("Still waiting to be sent")
+                raise TimeoutError("Still waiting to be sent")
             # wait for Future to indicate send having been called,
             # which means MessageTracker is ready.
             tic = time.time()
             if not self._sent_event.wait(timeout):
-                raise error.TimeoutError("Still waiting to be sent")
+                raise TimeoutError("Still waiting to be sent")
             if timeout:
                 timeout = max(0, timeout - (time.time() - tic))
         try:
@@ -444,7 +480,7 @@ class AsyncResult(Future):
                 timeout = -1
             return self._tracker.wait(timeout)
         except zmq.NotDone:
-            raise error.TimeoutError("Still waiting to be sent")
+            raise TimeoutError("Still waiting to be sent")
 
     # -------------------------------------
     # dict-access
@@ -477,7 +513,7 @@ class AsyncResult(Future):
         """getattr maps to getitem for convenient attr access to metadata."""
         try:
             return self.__getitem__(key)
-        except (error.TimeoutError, KeyError):
+        except (TimeoutError, KeyError):
             raise AttributeError(
                 "%r object has no attribute %r" % (self.__class__.__name__, key)
             )
@@ -497,7 +533,7 @@ class AsyncResult(Future):
             raise TypeError("AsyncResults with a single result are not iterable.")
         try:
             rlist = self.get(0)
-        except error.TimeoutError:
+        except TimeoutError:
             # wait for each result individually
             evt = Event()
             for child in self._children:
@@ -639,17 +675,34 @@ class AsyncResult(Future):
         ip.display_pub.publish(data=content['data'], metadata=md)
 
     def _display_stream(self, text, prefix='', file=None):
+        """Redisplay a stream"""
         if not text:
             # nothing to display
             return
         if file is None:
             file = sys.stdout
-        end = '' if text.endswith('\n') else '\n'
 
-        multiline = text.count('\n') > int(text.endswith('\n'))
-        if prefix and multiline and not text.startswith('\n'):
-            prefix = prefix + '\n'
-        print("%s%s" % (prefix, text), file=file, end=end)
+        end = ""
+        if prefix:
+            if prefix == self._last_display_prefix:
+                # same prefix, no need to re-display
+                prefix = ""
+            else:
+                self._last_display_prefix = prefix
+
+        if prefix and not self._stream_trailing_newline:
+            # prefix changed, no trailing newline; insert newline
+            pre = "\n"
+        else:
+            pre = ""
+
+        if prefix:
+            sep = "\n"
+        else:
+            sep = ""
+
+        self._stream_trailing_newline = text.endswith("\n")
+        print(f"{pre}{prefix}{sep}{text}", file=file, end="")
 
     def _display_single_result(self, result_only=False):
         if not result_only:
@@ -849,7 +902,7 @@ class AsyncMapResult(AsyncResult):
         """iterator for results *as they arrive*, preserving submission order."""
         try:
             rlist = self.get(0)
-        except error.TimeoutError:
+        except TimeoutError:
             # wait for each result individually
             evt = Event()
             for child in self._children:
@@ -866,7 +919,7 @@ class AsyncMapResult(AsyncResult):
         """iterator for results *as they arrive*, on FCFS basis, ignoring submission order."""
         try:
             rlist = self.get(0)
-        except error.TimeoutError:
+        except TimeoutError:
             queue = Queue()
             for child in self._children:
                 child.add_done_callback(queue.put)
