@@ -6,12 +6,14 @@ from __future__ import print_function
 import asyncio
 import json
 import os
+import re
 import socket
 import time
 import types
 import warnings
 from collections.abc import Iterable
 from concurrent.futures import Future
+from functools import partial
 from getpass import getpass
 from pprint import pprint
 from threading import current_thread
@@ -21,6 +23,7 @@ from threading import Thread
 import jupyter_client.session
 import zmq
 from decorator import decorator
+from ipykernel.comm import Comm
 from IPython import get_ipython
 from IPython.core.application import BaseIPythonApplication
 from IPython.core.profiledir import ProfileDir
@@ -35,6 +38,8 @@ from jupyter_client.session import Session
 from tornado import ioloop
 from traitlets import Any
 from traitlets import Bool
+from traitlets import Bytes
+from traitlets import default
 from traitlets import Dict
 from traitlets import HasTraits
 from traitlets import Instance
@@ -253,6 +258,10 @@ def _is_future(f):
     return hasattr(f, 'add_done_callback')
 
 
+# carriage return pattern
+_cr_pat = re.compile(r'.*\r(?=[^\n])')
+
+
 class Client(HasTraits):
     """A semi-synchronous client to an IPython parallel cluster
 
@@ -357,6 +366,11 @@ class Client(HasTraits):
     _connected = Bool(False)
     _ssh = Bool(False)
     _context = Instance('zmq.Context', allow_none=True)
+
+    @default("_context")
+    def _default_context(self):
+        return zmq.Context.instance()
+
     _config = Dict()
     _engines = Instance(util.ReverseDict, (), {})
     _query_socket = Instance('zmq.Socket', allow_none=True)
@@ -368,6 +382,10 @@ class Client(HasTraits):
     _broadcast_socket = Instance('zmq.Socket', allow_none=True)
     _registration_callbacks = List()
 
+    curve_serverkey = Bytes(allow_none=True)
+    curve_secretkey = Bytes(allow_none=True)
+    curve_publickey = Bytes(allow_none=True)
+
     _task_scheme = Unicode()
     _closed = False
 
@@ -399,9 +417,8 @@ class Client(HasTraits):
         if profile:
             super_kwargs['profile'] = profile
         super(Client, self).__init__(**super_kwargs)
-        if context is None:
-            context = zmq.Context.instance()
-        self._context = context
+        if context is not None:
+            self._context = context
 
         for argname in ('url_or_file', 'url_file'):
             if argname in extra_args:
@@ -468,6 +485,18 @@ class Client(HasTraits):
 
         self._task_scheme = cfg['task_scheme']
 
+        if not cfg.get("curve_serverkey") and "IPP_CURVE_SERVERKEY" in os.environ:
+            # load from env, if not set in connection file
+            cfg["curve_serverkey"] = os.environ["IPP_CURVE_SERVERKEY"]
+
+        if cfg.get("curve_serverkey"):
+            self.curve_serverkey = cfg["curve_serverkey"].encode('ascii')
+            if not self.curve_publickey or not self.curve_secretkey:
+                # if context: this could crash!
+                # inappropriately closes libsodium random_bytes source
+                # with libzmq <= 4.3.4
+                self.curve_publickey, self.curve_secretkey = zmq.curve_keypair()
+
         # sync defaults from args, json:
         if sshserver:
             cfg['ssh'] = sshserver
@@ -554,6 +583,10 @@ class Client(HasTraits):
         self.session = Session(**extra_args)
 
         self._query_socket = self._context.socket(zmq.DEALER)
+        if self.curve_serverkey:
+            self._query_socket.curve_serverkey = self.curve_serverkey
+            self._query_socket.curve_secretkey = self.curve_secretkey
+            self._query_socket.curve_publickey = self.curve_publickey
 
         if self._ssh:
             from zmq.ssh import tunnel
@@ -717,7 +750,13 @@ class Client(HasTraits):
 
                 return tunnel.tunnel_connection(s, url, sshserver, **ssh_kwargs)
             else:
-                return s.connect(url)
+                return util.connect(
+                    s,
+                    url,
+                    curve_serverkey=self.curve_serverkey,
+                    curve_secretkey=self.curve_secretkey,
+                    curve_publickey=self.curve_publickey,
+                )
 
         self.session.send(self._query_socket, 'connection_request')
         # use Poller because zmq.select has wrong units in pyzmq 2.1.7
@@ -726,7 +765,7 @@ class Client(HasTraits):
         # poll expects milliseconds, timeout is seconds
         evts = poller.poll(timeout * 1000)
         if not evts:
-            raise error.TimeoutError("Hub connection request timed out")
+            raise TimeoutError("Hub connection request timed out")
         idents, msg = self.session.recv(self._query_socket, mode=0)
         if self.debug:
             pprint(msg)
@@ -744,6 +783,7 @@ class Client(HasTraits):
             connect_socket(self._broadcast_socket, cfg['broadcast'])
 
             self._notification_socket = self._context.socket(zmq.SUB)
+            self._notification_socket.RCVHWM = 0
             self._notification_socket.setsockopt(zmq.SUBSCRIBE, b'')
             connect_socket(self._notification_socket, cfg['notification'])
 
@@ -751,6 +791,7 @@ class Client(HasTraits):
             connect_socket(self._control_socket, cfg['control'])
 
             self._iopub_socket = self._context.socket(zmq.SUB)
+            self._iopub_socket.RCVHWM = 0
             self._iopub_socket.setsockopt(zmq.SUBSCRIBE, b'')
             connect_socket(self._iopub_socket, cfg['iopub'])
 
@@ -794,6 +835,12 @@ class Client(HasTraits):
 
         if md['engine_uuid'] is not None:
             md['engine_id'] = self._engines.get(md['engine_uuid'], None)
+
+        if md['is_coalescing']:
+            # get destinations from target metadata
+            targets = msg_meta.get("broadcast_targets", [])
+            md['engine_uuid'], md['engine_id'] = map(list, zip(*targets))
+
         if 'date' in parent:
             md['submitted'] = parent['date']
         if 'started' in msg_meta:
@@ -808,8 +855,10 @@ class Client(HasTraits):
         eid = content['id']
         d = {eid: content['uuid']}
         self._update_engines(d)
+        event = {'event': 'register'}
+        event.update(content)
         for callback in self._registration_callbacks:
-            callback(content)
+            callback(event)
 
     def _unregister_engine(self, msg):
         """Unregister an engine that has died."""
@@ -824,6 +873,11 @@ class Client(HasTraits):
         if self._task_socket and self._task_scheme == 'pure':
             self._stop_scheduling_tasks()
 
+        event = {"event": "unregister"}
+        event.update(content)
+        for callback in self._registration_callbacks:
+            callback(event)
+
     def _handle_stranded_msgs(self, eid, uuid):
         """Handle messages known to be on an engine when the engine unregisters.
 
@@ -857,7 +911,11 @@ class Client(HasTraits):
         """
 
         parent = msg['parent_header']
-        msg_id = parent['msg_id']
+        if self._should_use_metadata_msg_id(msg):
+            msg_id = msg['metadata']['original_msg_id']
+        else:
+            msg_id = parent['msg_id']
+
         future = self._futures.get(msg_id, None)
         if msg_id not in self.outstanding:
             if msg_id in self.history:
@@ -874,9 +932,16 @@ class Client(HasTraits):
         md = self.metadata[msg_id]
         md.update(self._extract_metadata(msg))
 
-        e_outstanding = self._outstanding_dict[md['engine_uuid']]
-        if msg_id in e_outstanding:
-            e_outstanding.remove(msg_id)
+        if md['is_coalescing']:
+            engine_uuids = md['engine_uuid'] or []
+        else:
+            engine_uuids = [md['engine_uuid']]
+
+        for engine_uuid in engine_uuids:
+            if engine_uuid is not None:
+                e_outstanding = self._outstanding_dict[engine_uuid]
+                if msg_id in e_outstanding:
+                    e_outstanding.remove(msg_id)
 
         # construct result:
         if content['status'] == 'ok':
@@ -929,9 +994,16 @@ class Client(HasTraits):
         md = self.metadata[msg_id]
         md.update(self._extract_metadata(msg))
 
-        e_outstanding = self._outstanding_dict[md['engine_uuid']]
-        if msg_id in e_outstanding:
-            e_outstanding.remove(msg_id)
+        if md['is_coalescing']:
+            engine_uuids = md['engine_uuid'] or []
+        else:
+            engine_uuids = [md['engine_uuid']]
+
+        for engine_uuid in engine_uuids:
+            if engine_uuid is not None:
+                e_outstanding = self._outstanding_dict[engine_uuid]
+                if msg_id in e_outstanding:
+                    e_outstanding.remove(msg_id)
 
         # construct result:
         if content['status'] == 'ok':
@@ -1077,10 +1149,14 @@ class Client(HasTraits):
         # init metadata:
         md = self.metadata[msg_id]
 
+        ip = get_ipython()
+
         if msg_type == 'stream':
             name = content['name']
-            s = md[name] or ''
-            md[name] = s + content['text']
+            new_text = (md[name] or '') + content['text']
+            if '\r' in content['text']:
+                new_text = _cr_pat.sub('', new_text)
+            md[name] = new_text
         elif msg_type == 'error':
             md.update({'error': self._unwrap_exception(content)})
         elif msg_type == 'execute_input':
@@ -1099,9 +1175,54 @@ class Client(HasTraits):
                 if future and not future.done():
                     # TODO: should probably store actual outputs on the Future
                     future.set_result(None)
-        else:
-            # unhandled msg_type (status, etc.)
-            pass
+        elif msg_type.startswith("comm_") and ip is not None and ip.kernel is not None:
+            # only handle comm messages when we're in an IPython kernel
+            if msg_type == "comm_open":
+                # create proxy comm
+                engine_uuid = msg['metadata'].get('engine', '')
+                engine_ident = engine_uuid.encode("utf8", "replace")
+                # DEBUG: engine_uuid can still be missing?!
+
+                comm = Comm(
+                    comm_id=content['comm_id'],
+                    primary=False,
+                )
+
+                send_to_engine = partial(
+                    self._send,
+                    self._mux_socket,
+                    ident=engine_ident,
+                )
+
+                def relay_comm(msg):
+                    send_to_engine(
+                        msg["msg_type"],
+                        content=msg['content'],
+                        metadata=msg['metadata'],
+                        buffers=msg["buffers"],
+                    )
+
+                comm.on_msg(relay_comm)
+                comm.on_close(
+                    lambda: send_to_engine(
+                        "comm_close",
+                        content={
+                            "comm_id": comm.comm_id,
+                        },
+                    )
+                )
+                ip.kernel.comm_manager.register_comm(comm)
+
+            # relay all comm msgs
+            ip.kernel.session.send(
+                ip.kernel.iopub_socket,
+                msg_type,
+                content=msg['content'],
+                metadata=msg['metadata'],
+                buffers=msg['buffers'],
+                # different parent!
+                parent=ip.kernel.get_parent("shell"),
+            )
 
         msg_future = self._futures.get(msg_id, None)
         if msg_future:
@@ -1148,7 +1269,9 @@ class Client(HasTraits):
         )
         msg_id = msg['header']['msg_id']
 
-        if track_outstanding:
+        expect_reply = msg_type not in {"comm_msg", "comm_close", "comm_open"}
+
+        if expect_reply and track_outstanding:
             # add to outstanding, history
             self.outstanding.add(msg_id)
             self.history.append(msg_id)
@@ -1164,22 +1287,23 @@ class Client(HasTraits):
                     self._outstanding_dict[ident_str].add(msg_id)
             self.metadata['submitted'] = util.utcnow()
 
-        futures = self.create_message_futures(
-            msg_id,
-            async_result=msg_type in {'execute_request', 'apply_request'},
-            track=track,
-        )
-        if message_future_hook is not None:
-            message_future_hook(futures[0])
+        if expect_reply:
+            futures = self.create_message_futures(
+                msg_id,
+                async_result=msg_type in {'execute_request', 'apply_request'},
+                track=track,
+            )
+            if message_future_hook is not None:
+                message_future_hook(futures[0])
 
-        def cleanup(f):
-            """Purge caches on Future resolution"""
-            self.results.pop(msg_id, None)
-            self._futures.pop(msg_id, None)
-            self._output_futures.pop(msg_id, None)
-            self.metadata.pop(msg_id, None)
+            def cleanup(f):
+                """Purge caches on Future resolution"""
+                self.results.pop(msg_id, None)
+                self._futures.pop(msg_id, None)
+                self._output_futures.pop(msg_id, None)
+                self.metadata.pop(msg_id, None)
 
-        multi_future(futures).add_done_callback(cleanup)
+            multi_future(futures).add_done_callback(cleanup)
 
         def _really_send():
             sent = self.session.send(
@@ -1190,7 +1314,8 @@ class Client(HasTraits):
 
         # hand off actual send to IO thread
         self._io_loop.add_callback(_really_send)
-        return futures[0]
+        if expect_reply:
+            return futures[0]
 
     def _send_recv(self, *args, **kwargs):
         """Send a message in the IO thread and return its reply"""
@@ -1383,11 +1508,48 @@ class Client(HasTraits):
                 unit='engine',
             )
 
+        # watch for engine-stop events
+
+        engine_stop_future = Future()
+        if self.cluster and self.cluster.engines:
+            # we have a parent cluster,
+            # monitor for engines stopping
+            def _signal_stopped(stop_data):
+                if not engine_stop_future.done():
+                    engine_stop_future.set_result(stop_data)
+
+            for es in self.cluster.engines.values():
+                es.on_stop(_signal_stopped)
+                engine_stop_future.add_done_callback(
+                    lambda f: es.stop_callbacks.remove(_signal_stopped)
+                )
+
         future = Future()
 
-        def notify(_):
+        def cancel_engine_stop(_):
+            if not engine_stop_future.done():
+                engine_stop_future.cancel()
+
+        future.add_done_callback(cancel_engine_stop)
+
+        def notice_engine_stop(f):
             if future.done():
                 return
+            stop_data = f.result()
+            future.set_exception(error.EngineError(f"Engine set stopped: {stop_data}"))
+
+        engine_stop_future.add_done_callback(notice_engine_stop)
+
+        def notify(event):
+            if future.done():
+                return
+            if event["event"] == "unregister":
+                future.set_exception(
+                    error.EngineError(
+                        f"Engine {event['id']} unregistered while waiting for engines."
+                    )
+                )
+                return
             current_n = len(self.ids)
             if interactive:
                 progress_bar.update(current_n - progress_bar.n)
@@ -1397,8 +1559,8 @@ class Client(HasTraits):
                     progress_bar.close()
                 future.set_result(None)
 
-        future.add_done_callback(lambda f: self._registration_callbacks.remove(notify))
         self._registration_callbacks.append(notify)
+        future.add_done_callback(lambda f: self._registration_callbacks.remove(notify))
 
         def on_timeout():
             """Called when timeout is reached"""
