@@ -868,6 +868,17 @@ class DirectView(View):
         ip.magics_manager.register(M)
 
 
+@decorator
+def _not_coalescing(method, self, *args, **kwargs):
+    """Decorator for broadcast methods that can't use reply coalescing"""
+    is_coalescing = self.is_coalescing
+    try:
+        self.is_coalescing = False
+        return method(self, *args, **kwargs)
+    finally:
+        self.is_coalescing = is_coalescing
+
+
 class BroadcastView(DirectView):
     is_coalescing = Bool(False)
 
@@ -962,6 +973,7 @@ class BroadcastView(DirectView):
 
     @sync_results
     @save_ids
+    @_not_coalescing
     def execute(self, code, silent=True, targets=None, block=None):
         """Executes `code` on `targets` in blocking or nonblocking manner.
 
@@ -1010,6 +1022,10 @@ class BroadcastView(DirectView):
     def map(self, f, *sequences, **kwargs):
         raise NotImplementedError("BroadcastView.map not yet implemented")
 
+    # scatter/gather cannot be coalescing yet
+    scatter = _not_coalescing(DirectView.scatter)
+    gather = _not_coalescing(DirectView.gather)
+
 
 class LoadBalancedView(View):
     """An load-balancing View that only executes via the Task scheduler.
@@ -1462,10 +1478,7 @@ class LoadBalancedView(View):
         )
 
 
-from concurrent.futures import Executor
-
-
-class ViewExecutor(Executor):
+class ViewExecutor(concurrent.futures.Executor):
     """A PEP-3148 Executor API for Views
 
     Access as view.executor
@@ -1473,6 +1486,7 @@ class ViewExecutor(Executor):
 
     def __init__(self, view):
         self.view = view
+        self._max_workers = len(self.view)
 
     def submit(self, fn, *args, **kwargs):
         """Same as View.apply_async"""
