@@ -351,8 +351,16 @@ class ControllerLauncher(BaseLauncher):
         self.log.debug(f"Loading {paths}")
         connection_info = {}
         for key, path in connection_files.items():
-            with open(path) as f:
-                connection_info[key] = json.load(f)
+            try:
+                with open(path) as f:
+                    connection_info[key] = json.load(f)
+            except ValueError:
+                # possible race while controller is still writing the file
+                # give it half a second before trying again
+                time.sleep(0.5)
+                with open(path) as f:
+                    connection_info[key] = json.load(f)
+
         return connection_info
 
 
@@ -1048,12 +1056,16 @@ class SSHLauncher(LocalProcessLauncher):
 
     @default("remote_output_file")
     def _default_remote_output_file(self):
-        if 'engine' in ' '.join(self.program):
+        full_program = ' '.join(self.program)
+        if 'engine' in full_program:
             name = 'ipengine'
+        elif 'controller' in full_program:
+            name = 'ipcontroller'
         else:
             name = self.program[0]
         return os.path.join(
             self.remote_profile_dir,
+            'log',
             os.path.basename(name) + f"-{time.time():.4f}.out",
         )
 
@@ -1110,6 +1122,13 @@ class SSHLauncher(LocalProcessLauncher):
         if 'pid' in d and d['pid'] > 0:
             self._start_waiting()
 
+    def poll(self):
+        """Override poll"""
+        if self.state == 'running':
+            return None
+        else:
+            return 0
+
     def get_output(self, remove=False):
         """Retrieve engine output from the remote file"""
         if self._output is None:
