@@ -1,25 +1,24 @@
-try:
-    from unittest import mock
-except ImportError:
-    import mock
+from unittest import mock
 
 import pytest
 
 import ipyparallel as ipp
-from .clienttest import ClusterTestCase, add_engines
+from .clienttest import add_engines
+from .clienttest import ClusterTestCase
 
 try:
-    import joblib
     from joblib import Parallel, delayed
-    from ipyparallel.client._joblib import IPythonParallelBackend
+    from ipyparallel.client._joblib import IPythonParallelBackend  # noqa
 except (ImportError, TypeError):
     have_joblib = False
 else:
     have_joblib = True
 
+
 def neg(x):
     return -1 * x
 
+
 class TestJobLib(ClusterTestCase):
     def setUp(self):
         if not have_joblib:
@@ -33,9 +32,7 @@ class TestJobLib(ClusterTestCase):
         with mock.patch.object(ipp.Client, "__new__", lambda *a, **kw: self.client):
             p = Parallel(backend='ipyparallel')
             assert p._backend._view.client is self.client
-        
-        self.client[:].use_pickle()
-    
+
     def test_register_backend(self):
         view = self.client.load_balanced_view()
         view.register_joblib_backend('view')
