@@ -1,10 +1,10 @@
 import socket
-try:
-    from unittest import mock
-except ImportError:
-    import mock
+from unittest import mock
+
+from jupyter_client.localinterfaces import localhost
+from jupyter_client.localinterfaces import public_ips
+
 from ipyparallel import util
-from jupyter_client.localinterfaces import localhost, public_ips
 
 
 @mock.patch('warnings.warn')
@@ -18,6 +18,6 @@ def test_disambiguate_ip(warn_mock):
     assert warn_mock.called_once_with(
         'IPython could not determine IPs for {}: '
         '[Errno -2] Name or service not known'.format(wontresolve),
-        RuntimeWarning
+        RuntimeWarning,
     )
     assert util.disambiguate_ip_address('0.0.0.0', public_ip) == localhost()
